@@ -2,7 +2,7 @@
 
 #  FLAC - Free Lossless Audio Codec
 #  Copyright (C) 2001-2009  Josh Coalson
-#  Copyright (C) 2011-2022  Xiph.Org Foundation
+#  Copyright (C) 2011-2025  Xiph.Org Foundation
 #
 #  This file is part the FLAC project.  FLAC is comprised of several
 #  components distributed under different licenses.  The codec libraries
@@ -182,6 +182,20 @@ rt_test_wav ()
 	rm -f rt.flac rt.wav
 }
 
+rt_test_wav_autokf ()
+{
+	f="$1"
+	extra="$2"
+	echo $ECHO_N "round-trip test ($f) encode... " $ECHO_C
+	run_flac --force --verify --channel-map=none --no-padding --lax -o rt.flac $extra $f || die "ERROR"
+	echo $ECHO_N "decode... " $ECHO_C
+	run_flac --force --decode --channel-map=none $extra rt.flac || die "ERROR"
+	echo $ECHO_N "compare... " $ECHO_C
+	cmp $f rt.wav || die "ERROR: file mismatch"
+	echo "OK"
+	rm -f rt.flac rt.wav
+}
+
 rt_test_w64 ()
 {
 	f="$1"
@@ -196,6 +210,20 @@ rt_test_w64 ()
 	rm -f rt.flac rt.w64
 }
 
+rt_test_w64_autokf ()
+{
+	f="$1"
+	extra="$2"
+	echo $ECHO_N "round-trip test ($f) encode... " $ECHO_C
+	run_flac --force --verify --channel-map=none --no-padding --lax -o rt.flac $extra $f || die "ERROR"
+	echo $ECHO_N "decode... " $ECHO_C
+	run_flac --force --decode --channel-map=none $extra rt.flac || die "ERROR"
+	echo $ECHO_N "compare... " $ECHO_C
+	cmp $f rt.w64 || die "ERROR: file mismatch"
+	echo "OK"
+	rm -f rt.flac rt.w64
+}
+
 rt_test_rf64 ()
 {
 	f="$1"
@@ -210,6 +238,20 @@ rt_test_rf64 ()
 	rm -f rt.flac rt.rf64
 }
 
+rt_test_rf64_autokf ()
+{
+	f="$1"
+	extra="$2"
+	echo $ECHO_N "round-trip test ($f) encode... " $ECHO_C
+	run_flac --force --verify --channel-map=none --no-padding --lax -o rt.flac $extra $f || die "ERROR"
+	echo $ECHO_N "decode... " $ECHO_C
+	run_flac --force --decode --channel-map=none $extra rt.flac || die "ERROR"
+	echo $ECHO_N "compare... " $ECHO_C
+	cmp $f rt.rf64 || die "ERROR: file mismatch"
+	echo "OK"
+	rm -f rt.flac rt.rf64
+}
+
 rt_test_aiff ()
 {
 	f="$1"
@@ -224,6 +266,48 @@ rt_test_aiff ()
 	rm -f rt.flac rt.aiff
 }
 
+rt_test_aifc ()
+{
+	f="$1"
+	extra="$2"
+	echo $ECHO_N "round-trip test ($f) encode... " $ECHO_C
+	run_flac --force --verify --channel-map=none --no-padding --lax -o rt.flac $extra $f || die "ERROR"
+	echo $ECHO_N "decode... " $ECHO_C
+	run_flac --force --decode --channel-map=none -o rt.aifc --force-aiff-c-none-format $extra rt.flac || die "ERROR"
+	echo $ECHO_N "compare... " $ECHO_C
+	cmp $f rt.aifc || die "ERROR: file mismatch"
+	echo "OK"
+	rm -f rt.flac rt.aifc
+}
+
+rt_test_aifc_le ()
+{
+	f="$1"
+	extra="$2"
+	echo $ECHO_N "round-trip test ($f) encode... " $ECHO_C
+	run_flac --force --verify --channel-map=none --no-padding --lax -o rt.flac $extra $f || die "ERROR"
+	echo $ECHO_N "decode... " $ECHO_C
+	run_flac --force --decode --channel-map=none -o rt.aifc --force-aiff-c-sowt-format $extra rt.flac || die "ERROR"
+	echo $ECHO_N "compare... " $ECHO_C
+	cmp $f rt.aifc || die "ERROR: file mismatch"
+	echo "OK"
+	rm -f rt.flac rt.aifc
+}
+
+rt_test_autokf ()
+{
+	f="$1"
+	extra="$2"
+	echo $ECHO_N "round-trip test ($f) encode... " $ECHO_C
+	run_flac --force --verify --no-padding --lax -o rt.flac $extra $f || die "ERROR"
+	echo $ECHO_N "decode... " $ECHO_C
+	run_flac --force --decode $extra rt.flac || die "ERROR"
+	echo $ECHO_N "compare... " $ECHO_C
+	cmp $f $3 || die "ERROR: file mismatch"
+	echo "OK"
+	rm -f rt.flac $3
+}
+
 # assumes input file is WAVE; does not check the metadata-preserving features of flac-to-flac; that is checked later
 rt_test_flac ()
 {
@@ -273,6 +357,12 @@ done
 for f in rt-*.aiff ; do
 	rt_test_aiff $f
 done
+for f in rt-*[0-9].aifc ; do
+	rt_test_aifc $f
+done
+for f in rt-*le.aifc ; do
+	rt_test_aifc_le $f
+done
 for f in rt-*.wav ; do
 	rt_test_flac $f
 done
@@ -1036,45 +1126,12 @@ cp shortnoise.raw file0.raw
 cp shortnoise.raw file1.raw
 cp shortnoise.raw file2.raw
 rm -f shortnoise.raw
-# create authoritative sector-aligned files for comparison
-file0_samples=$(( (short_noise_cdda_samples / 588) * 588))
-file0_remainder=$((short_noise_cdda_samples - file0_samples))
-file1_samples=$(( ( ( file0_remainder + short_noise_cdda_samples ) / 588 ) * 588))
-file1_remainder=$((file0_remainder + short_noise_cdda_samples - file1_samples))
-file1_samples=$((file1_samples - file0_remainder))
-file2_samples=$(( ( ( file1_remainder + short_noise_cdda_samples ) / 588 ) * 588))
-file2_remainder=$(( file1_remainder + short_noise_cdda_samples - file2_samples))
-file2_samples=$((file2_samples - file1_remainder))
-if [ $file2_remainder != '0' ] ; then
-	file2_samples=$((file2_samples + file2_remainder))
-	file2_remainder=$((588 - file2_remainder))
-fi
-
-dd if=file0.raw ibs=4 count=$file0_samples of=file0s.raw 2>/dev/null || $dddie
-dd if=file0.raw ibs=4 count=$file0_remainder of=file1s.raw skip=$file0_samples 2>/dev/null || $dddie
-dd if=file1.raw ibs=4 count=$file1_samples of=z.raw 2>/dev/null || $dddie
-cat z.raw >> file1s.raw || die "ERROR: cat-ing sector-aligned files"
-dd if=file1.raw ibs=4 count=$file1_remainder of=file2s.raw skip=$file1_samples 2>/dev/null || $dddie
-dd if=file2.raw ibs=4 count=$file2_samples of=z.raw 2>/dev/null || $dddie
-cat z.raw >> file2s.raw || die "ERROR: cat-ing sector-aligned files"
-dd if=/dev/zero ibs=4 count=$file2_remainder of=z.raw 2>/dev/null || $dddie
-cat z.raw >> file2s.raw || die "ERROR: cat-ing sector-aligned files"
-rm -f z.raw
-
-convert_to_wav file0s "$multifile_format_encode --force --force-raw-format" "$SILENT --force --decode" || die "ERROR creating authoritative sector-aligned WAVE"
-convert_to_wav file1s "$multifile_format_encode --force --force-raw-format" "$SILENT --force --decode" || die "ERROR creating authoritative sector-aligned WAVE"
-convert_to_wav file2s "$multifile_format_encode --force --force-raw-format" "$SILENT --force --decode" || die "ERROR creating authoritative sector-aligned WAVE"
-
-convert_to_aiff file0s "$multifile_format_encode --force --force-raw-format" "$SILENT --force --decode" || die "ERROR creating authoritative sector-aligned AIFF"
-convert_to_aiff file1s "$multifile_format_encode --force --force-raw-format" "$SILENT --force --decode" || die "ERROR creating authoritative sector-aligned AIFF"
-convert_to_aiff file2s "$multifile_format_encode --force --force-raw-format" "$SILENT --force --decode" || die "ERROR creating authoritative sector-aligned AIFF"
 
 test_multifile ()
 {
 	input_type=$1
 	streamtype=$2
-	sector_align=$3
-	encode_options="$4"
+	encode_options="$3"
 
 	extra_encode_options=""
 	extra_decode_options=""
@@ -1094,10 +1151,6 @@ test_multifile ()
 		suffix=flac
 	fi
 
-	if [ $sector_align = sector_align ] ; then
-		encode_options="$encode_options --sector-align"
-	fi
-
 	if [ $input_type = flac ] || [ $input_type = ogg ] ; then
 		CMP=md5cmp
 	else
@@ -1109,15 +1162,9 @@ test_multifile ()
 	done
 	run_flac --force $encode_options $extra_encode_options file0x.$input_type file1x.$input_type file2x.$input_type || die "ERROR"
 	run_flac --force --decode $extra_decode_options file0x.$suffix file1x.$suffix file2x.$suffix || die "ERROR"
-	if [ $sector_align != sector_align ] ; then
-		for n in 0 1 2 ; do
-			$CMP file$n.$input_type file${n}x.$input_type || die "ERROR: file mismatch on file #$n"
-		done
-	else
-		for n in 0 1 2 ; do
-			$CMP file${n}s.$input_type file${n}x.$input_type || die "ERROR: file mismatch on file #$n"
-		done
-	fi
+	for n in 0 1 2 ; do
+		$CMP file$n.$input_type file${n}x.$input_type || die "ERROR: file mismatch on file #$n"
+	done
 	for n in 0 1 2 ; do
 		rm -f file${n}x.$suffix file${n}x.$input_type
 	done
@@ -1130,36 +1177,20 @@ input_types="raw wav aiff flac"
 #@@@fi
 for input_type in $input_types ; do
 	echo "Testing multiple $input_type files without verify..."
-	test_multifile $input_type flac no_sector_align ""
+	test_multifile $input_type flac ""
 
 	echo "Testing multiple $input_type files with verify..."
-	test_multifile $input_type flac no_sector_align "--verify"
-
-	if [ $input_type != flac ] && [ $input_type != ogg ] ; then # --sector-align not supported for FLAC input
-		echo "Testing multiple $input_type files with --sector-align, without verify..."
-		test_multifile $input_type flac sector_align ""
-
-		echo "Testing multiple $input_type files with --sector-align, with verify..."
-		test_multifile $input_type flac sector_align "--verify"
-	fi
+	test_multifile $input_type flac "--verify"
 
 	if [ $has_ogg = yes ] ; then
 		echo "Testing multiple $input_type files with --ogg, without verify..."
-		test_multifile $input_type ogg no_sector_align ""
+		test_multifile $input_type ogg ""
 
 		echo "Testing multiple $input_type files with --ogg, with verify..."
-		test_multifile $input_type ogg no_sector_align "--verify"
-
-		if [ $input_type != flac ] ; then # --sector-align not supported for FLAC input
-			echo "Testing multiple $input_type files with --ogg and --sector-align, without verify..."
-			test_multifile $input_type ogg sector_align ""
-
-			echo "Testing multiple $input_type files with --ogg and --sector-align, with verify..."
-			test_multifile $input_type ogg sector_align "--verify"
-		fi
+		test_multifile $input_type ogg "--verify"
 
 		echo "Testing multiple $input_type files with --ogg and --serial-number, with verify..."
-		test_multifile $input_type ogg no_sector_align "--serial-number=321 --verify"
+		test_multifile $input_type ogg "--serial-number=321 --verify"
 	fi
 done
 
@@ -1177,6 +1208,20 @@ rt_test_w64 wacky2.w64 '--keep-foreign-metadata'
 rt_test_rf64 wacky1.rf64 '--keep-foreign-metadata'
 rt_test_rf64 wacky2.rf64 '--keep-foreign-metadata'
 
+rt_test_wav_autokf wacky1.wav '--keep-foreign-metadata'
+rt_test_wav_autokf wacky2.wav '--keep-foreign-metadata'
+rt_test_w64_autokf wacky1.w64 '--keep-foreign-metadata'
+rt_test_w64_autokf wacky2.w64 '--keep-foreign-metadata'
+rt_test_rf64_autokf wacky1.rf64 '--keep-foreign-metadata'
+rt_test_rf64_autokf wacky2.rf64 '--keep-foreign-metadata'
+
+testdatadir=${top_srcdir}/test/foreign-metadata-test-files
+
+rt_test_autokf "$testdatadir/BWF-WaveFmtEx.wav" '--keep-foreign-metadata' 'rt.wav'
+rt_test_autokf "$testdatadir/AIFF-ID3.aiff" '--keep-foreign-metadata' 'rt.aiff'
+rt_test_autokf "$testdatadir/AIFF-C-sowt-tag.aifc" '--keep-foreign-metadata' 'rt.aifc'
+rt_test_autokf "$testdatadir/AIFF-C-sowt-compression-type-name.aifc" '--keep-foreign-metadata' 'rt.aifc'
+rt_test_autokf "$testdatadir/24bit-WaveFmtPCM.wav" '--keep-foreign-metadata' 'rt.wav'
 
 ############################################################################
 # test the metadata-handling properties of flac-to-flac encoding
@@ -1264,6 +1309,20 @@ fi
 
 echo OK
 
+############################################################################
+# test threads
+############################################################################
+
+
+for J in 1 2 3 4 5 6 7 8 15 16 32 63 64; do
+	echo $ECHO_N "Testing --threads $J... " $ECHO_C
+	run_flac -f -V -o out.flac --threads $J "noisy-sine.wav" || die "ERROR on encoding"
+	run_flac -f -d out.flac || die "ERROR on decoding"
+	echo OK
+done
+
+rm -f out.wav
+
 ############################################################################
 # test overflow of total samples field in STREAMINFO
 ############################################################################
